%
%  Subject : Discretization techniques using narrow-base trial functions 
%
%  Example Ej_4_3
%
%           approximation in 1D : heat equation as a system of two 1st
%           order ODEs
%           example 3.3 , page 114, Zienkiewicz & Morgan
%

% Number of elements to be used
Nx = 4;

% boundary conditions
phi_lef = 0;  % phi(x=0)
q_rig   = 0;  % q(x=1)
ndf = 2;

phi_0 = zeros(ndf*(Nx+1),1);

% grid points and mesh
xnod  = (0:Nx)'/Nx;
icone = [(1:Nx)',(2:Nx+1)'];

[numel,nen]=size(icone);

Ke = zeros(numel,nen,ndf,nen,ndf);
fe = zeros(numel,nen,ndf);

filename = 'ffun_Ej_4_3_Q';
kappa = 1;

% loop over each element
for k=1:numel
    psi = (0:20)'/20; 
    node1 = icone(k,1);
    node2 = icone(k,2);
    xx  = psi*(xnod(node2,1)-xnod(node1,1))+xnod(node1,1);            
    eval(['Qx=' filename '(xx);' ]);
    [N,L_N] = shape_function(xx);        
    N1 = N; N2 = N;
    Nq = N; Nphi = N;
    L_Nq = L_N; L_Nphi = L_N;    
    for mi=1:nen        
        for mj=1:nen            
            Ke(k,mi,1,mj,1) = Ke(k,mi,1,mj,1) + trapz(xx,N1(:,mi).*Nq(:,mj)) ;    
            Ke(k,mi,1,mj,2) = Ke(k,mi,1,mj,2) + kappa*trapz(xx,N1(:,mi).*L_Nphi(:,mj)) ;                
            Ke(k,mi,2,mj,1) = Ke(k,mi,2,mj,1) + trapz(xx,N2(:,mi).*L_Nq(:,mj)) ;                            
        end
        fe(k,mi,2) = fe(k,mi,2) + trapz(xx,N2(:,mi).*Qx) ; 
    end    
end

% gather Ke and fe in Kg and fg
Kg = zeros(ndf*(Nx+1),ndf*(Nx+1));
fg = zeros(ndf*(Nx+1),1);
for k=1:numel
    for mi=1:nen
        node_i = icone(k,mi);
        for mj=1:nen
            node_j=icone(k,mj);
            Kg((node_i-1)*ndf+(1:ndf),(node_j-1)*ndf+(1:ndf))=Kg((node_i-1)*ndf+(1:ndf),(node_j-1)*ndf+(1:ndf)) + reshape(Ke(k,mi,:,mj,:),ndf,ndf);        
        end
        fg((node_i-1)*ndf+(1:ndf),1) = fg((node_i-1)*ndf+(1:ndf),1) + reshape(fe(k,mi,:),ndf,1);
    end
end

% reducing the global system for fixations
free = (1:ndf*(Nx+1))';
free([2,ndf*(Nx)+1])=[];
fg(free,1) = fg(free,1) - Kg(free,:)*phi_0 ;
Kg = Kg(free,free);
fg = fg(free,1);
% solver
phi_new = Kg\fg;

phi = [phi_new(1);phi_0(2);phi_new(2:end-1);phi_0(ndf*(Nx)+1);phi_new(end)];

% Visualization
figure(1);clf;
x = xnod(:,1);
xe = 0.5*(x(1:end-1)+x(2:end));
plot(x,zeros(size(x)),'bo',x,-phi(1:ndf:end),'k-',x,phi(2:ndf:end),'r-')
xlabel('X');ylabel('(-q,phi)');title('1D heat equation with piecewise linear Galerkin aproximation as a system of 1st order ODEs ')
%errphi = norm(phi-phi_th);

if 1
    xnod=[ [xnod , 0*xnod] ; [xnod , max(xnod)*0.1*ones(size(xnod,1),1)] ];
    icone = [icone , icone(:,[2,1])+Nx+1];
    figure(2);clf;pltmsh(xnod,icone);pfnodes(xnod,icone);axis equal
end